/*
 * Name: buffer.c
 * Project: PassKeychain
 * Author: Jiri Stepanovsky - http://jiristepanovsky.cz
 * Creation Date: 2014-11-28
 * Tabsize: 4
 * License: MIT License
 */


#include "buffer.h"

/* Buffer pointers */
volatile unsigned char bufferWritePointer, bufferReadPointer;

/* Actual buffers */
volatile unsigned char buff[BUFFER_SIZE];

/* Reset both buffers */
void bufferReset(void){
	bufferWritePointer = 0;
	bufferReadPointer = 0;
}

/* Access to Transmit buffer */
void bufferWrite(unsigned char data){
	unsigned char next = bufferWritePointer + 1;
	if(next>=(BUFFER_SIZE & 0xFF))
		next = 0;

	if(next != bufferReadPointer){
		buff[bufferWritePointer] = data;
		bufferWritePointer = next;
	} // else { overflow }
}

unsigned char bufferRead(void){
	unsigned char data = 0;
	if(bufferWritePointer != bufferReadPointer){
		data = buff[bufferReadPointer];
		bufferReadPointer++;
		if(bufferReadPointer>=(BUFFER_SIZE&0xFF))
			bufferReadPointer = 0;
	}
	return data;
}

unsigned char bufferSize(void){
	unsigned char size = 0;
	if(bufferWritePointer > bufferReadPointer){
		size = bufferWritePointer - bufferReadPointer;
	} else if (bufferWritePointer < bufferReadPointer){
		size = (BUFFER_SIZE - bufferReadPointer) + bufferWritePointer;
	}
	return size;
}

unsigned char bufferFree(void){
	unsigned char size = 0;
	if(bufferWritePointer >= bufferReadPointer){
		size = ((BUFFER_SIZE-1) - bufferWritePointer) + bufferReadPointer;
	} else if (bufferWritePointer < bufferReadPointer){
		size = (bufferReadPointer - bufferWritePointer) - 1;
	}
	return size;
}
