#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QGraphicsScene>
#include <QGraphicsItem>
#include <QThread>
#include <QResizeEvent>
#include <QPixmap>
#include <QImage>
#include <QTimer>
//#include <QFile>

#include "thread.h"
#include "fft.h"

#define CANVAS_MARGIN 4
#define CANVAS_MIN_SIZE_X 600
#define CANVAS_MIN_SIZE_Y 400
#define PAINT_MARGIN 5

#define FFT_OFFSET 20 // Left margin
#define FFT_STAGE_NUM 8 // Number of stages in memory
#define FFT_STAGE_SIZE 250 // Samples per 1 stage
#define FFT_PIXELS_PER_INDEX 2

#define SCOPE_HEIGHT 352
#define SCOPE_STACKED_POINTS 8 // Samples stacked on one vertical line

#define DATA_BUFFER_SIZE (FFT_STAGE_SIZE+100) // bit larger than fft stage size

#define GRAPHICS_UPDATE_MS 50

#define FILE_LOGGING 0
#define FILE_PATH "../../../log.txt"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

private slots:
    void resizeEvent(QResizeEvent* event);
    void resize_canvas();
    void on_info(const QString text);
    void scope(const QVector<int> data);
    void fft_update();
    void graphics_update();
    void on_pushButton_clicked();

private:
    Ui::MainWindow *ui;
    Thread *hid_thread;
    QGraphicsScene scene;
    QTimer graphics_timer;
    QImage canvas;
#if FILE_LOGGING
    QFile file_log;
#endif

    int width, height;

    int scope_x, scope_val;

    int fft_x;
    int fft_panel_height;

    int data_buffer_size = 0;
    int data_buffer[DATA_BUFFER_SIZE];

    int fft_stage_size[FFT_STAGE_NUM];
    int fft_buffer[FFT_STAGE_NUM*FFT_STAGE_SIZE];
    complex data_a[FFT_STAGE_NUM*FFT_STAGE_SIZE], data_b[FFT_STAGE_NUM*FFT_STAGE_SIZE];
};


#endif // MAINWINDOW_H
