#include "thread.h"

#ifdef Q_OS_WIN
#include <windows.h>
#endif

Thread::Thread()
{   
    device = nullptr;
}

Thread::~Thread()
{
}

void Thread::qSleep(int ms)
{
#ifdef Q_OS_WIN
    Sleep(uint(ms));
#else
    struct timespec ts = { ms / 1000, (ms % 1000) * 1000 * 1000 };
    nanosleep(&ts, NULL);
#endif
}

QByteArray Thread::hid_read_(void)
{
    QByteArray array;
    unsigned char buffer[8];
    int res = 0;
    buffer[0] = 0x01;
    do {
        if ((res = hid_get_feature_report(device, buffer, 9)) > 1) {
#if defined(Q_OS_WIN)
            --res;
#endif
            for (int i = 1; i < res; ++i)
                array.append(buffer[i]);
            connection_timeout = 0;
        } else {
            connection_timeout++;
            if (connection_timeout > CONNECTION_TIMEOUT_MS/DATA_POOL_INTERVAL_MS) {
                hid_close(device);
                device = nullptr;
                emit info("Device disconnected");
                qSleep(2000);
            }
        }
    } while (res > 1);
    return array;
}

QVector<int> Thread::read_data(void)
{
    QByteArray a = hid_read_();
    QVector<int> v;
    for (int i = 0; i < a.size(); i += 2) {
        if (last_byte != -1) {
            int value = last_byte | ((unsigned char)a.at(i));
            last_byte = -1;
            if (value > 1023) --i;
            else v.append(value);
        } else if (i+1 < a.size()) {
            int value = (((unsigned char)a.at(i))<<8) | ((unsigned char)a.at(i+1));
            if (value > 1023) --i;
            else v.append(value);
        } else
            last_byte = ((unsigned char)a.at(i))<<8;
    }
    return v;
}

void Thread::stop(void)
{
    running = 0;
    while (running >= 0) qSleep(10);
}

void Thread::run() {
    int data_incomming = 0;
    if (hid_init()) {
        emit info("HID API could not be initialized");
        running = -1;
        return;
    }
    QTime time;
    time.start();
    int timeout = 0;
    while(running) {
        if (device == nullptr) {
            struct hid_device_info *devs = hid_enumerate(0x16C0, 0x05DA);
            struct hid_device_info *cur_dev = devs;
            char *path = nullptr;
            while (cur_dev) {
                QString name;
                wchar_t *c = cur_dev->product_string;
                while (*c != 0)
                    name += *(c++);
                if (name == "USB-ECG") {
                    path = (char*) malloc(sizeof(char)*(strlen(cur_dev->path)+1));
                    strcpy(path, cur_dev->path);
                    break;
                }
                cur_dev = cur_dev->next;
            }
            hid_free_enumeration(devs);
            if (path == nullptr) {
                emit info("No USB-ECG device found");
                qSleep(2000);
            } else {
                device = hid_open_path(path);
                if (device == nullptr) {
                    emit info(QString("Unable to open device path: ")+path);
                    qSleep(2000);
                } else {
                    emit info("Connected");
                    data_incomming = 0;
                    time.restart();
                    timeout = 1000;
                    qSleep(DATA_POOL_INTERVAL_MS);
                }
            }
        } else {
            QVector<int> v = read_data();
            data_incomming += v.size();
            if (time.elapsed() > timeout) {
                timeout += 1000;
                if (timeout > 100000) { time.restart(); timeout = 1000;}
                emit info("Connected     (refresh rate: "+QString::number(data_incomming)+" Hz )");
                data_incomming = 0;
            }
            emit data(v);
            qSleep(DATA_POOL_INTERVAL_MS);
        }
    }
    if (device != nullptr)
        hid_close(device);
    hid_exit();
    running = -1;
}
